/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    Versaloon                                                 *
 *  File:       fifo.c                                                    *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    fifo implementation file                                  *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-11-07:     created(by SimonQian)                             *
 **************************************************************************/

#include "app_cfg.h"
#include "fifo.h"

//#define FIFO_Get_Next_Idx(pos, size)		++(pos) % (size)
static uint32 FIFO_Get_Next_Idx(uint32 pos, uint32 size)
{
	pos++;
	if(pos >= size)
	{
		pos = 0;
	}

	return pos;
}

uint32 FIFO_Get_Length(FIFO *fifo)
{
	return fifo->length;
}

uint32 FIFO_Add_Byte(FIFO *fifo, uint8 data)
{
	if ((fifo->size - fifo->length) > 0)
	{
		fifo->buffer[fifo->header] = data;
		fifo->header = FIFO_Get_Next_Idx(fifo->header, fifo->size);
		FIFO_DISABLE_READ_INT();
		fifo->length++;
		FIFO_RESTORE_READ_INT();
		return 1;
	}
	else
	{
		return 0;
	}
}

uint32 FIFO_Add_Buffer(FIFO *fifo, uint8 *src_buff, uint32 len)
{
	uint32 actual_len = len;
	if(actual_len > (fifo->size - fifo->length))
	{
		actual_len = fifo->size - fifo->length;
	}

	if(actual_len > (fifo->size - fifo->header))
	{
		memcpy(&fifo->buffer[fifo->header], &src_buff[0], fifo->size - fifo->header);
		memcpy(&fifo->buffer[0], &src_buff[fifo->size - fifo->header], actual_len - (fifo->size - fifo->header));
		fifo->header = actual_len - (fifo->size - fifo->header);
	}
	else
	{
		memcpy(&fifo->buffer[fifo->header], &src_buff[0], actual_len);
		fifo->header += actual_len;
		if(fifo->header == fifo->size)
		{
			fifo->header = 0;
		}
	}
	FIFO_DISABLE_READ_INT();
	fifo->length += actual_len;
	FIFO_RESTORE_READ_INT();

	return actual_len;
}

uint32 FIFO_Get_Buffer(FIFO *fifo, uint8 *dest_buff, uint32 len)
{
	uint32 actual_len = len;
	if(actual_len > fifo->length)
	{
		actual_len = fifo->length;
	}

	if(actual_len > (fifo->size - fifo->tail))
	{
		memcpy(&dest_buff[0], &fifo->buffer[fifo->tail], fifo->size - fifo->tail);
		memcpy(&dest_buff[fifo->size - fifo->tail], &fifo->buffer[0], actual_len - (fifo->size - fifo->tail));
		fifo->tail = actual_len - (fifo->size - fifo->tail);
	}
	else
	{
		memcpy(&dest_buff[0], &fifo->buffer[fifo->tail], actual_len);
		fifo->tail += actual_len;
		if(fifo->tail == fifo->size)
		{
			fifo->tail = 0;
		}
	}
	FIFO_DISABLE_WRITE_INT();
	fifo->length -= actual_len;
	FIFO_RESTORE_WRITE_INT();

	return actual_len;
}

// make sure there is data in the buffer, if no data is in the buffer, 0 is returned
uint8 FIFO_Get_Byte(FIFO *fifo)
{
	uint8 byte;

	if(fifo->length > 0)
	{
		byte = fifo->buffer[fifo->tail];
		fifo->tail = FIFO_Get_Next_Idx(fifo->tail, fifo->size);
		FIFO_DISABLE_WRITE_INT();
		fifo->length--;
		FIFO_RESTORE_WRITE_INT();

		return byte;
	}
	return 0;
}
